Index: apps/app_voicemail.c
===================================================================
--- apps/app_voicemail.c	(revision 27127)
+++ apps/app_voicemail.c	(working copy)
@@ -53,6 +53,7 @@
 #include <sys/mman.h>
 #include <time.h>
 #include <dirent.h>
+#include <grp.h>
 
 #include "asterisk.h"
 
@@ -426,6 +427,8 @@
 static char callcontext[AST_MAX_CONTEXT];
 static char exitcontext[AST_MAX_CONTEXT];
 
+static gid_t mbg;
+
 static char cidinternalcontexts[MAX_NUM_CID_CONTEXTS][64];
 
 
@@ -445,6 +448,13 @@
 
 LOCAL_USER_DECL;
 
+static void set_mbgroup(const char* pathname)
+{
+	if (chown(pathname, -1, mbg)) {
+		ast_log(LOG_WARNING, "chown '%s' failed: %s\n", pathname, strerror(errno));
+	}
+}
+				
 static void populate_defaults(struct ast_vm_user *vmu)
 {
 	ast_copy_flags(vmu, (&globalflags), AST_FLAGS_ALL);	
@@ -825,6 +835,7 @@
 			return 0;
 		}
 	}
+	set_mbgroup(dest);
 	return 1;
 }
 
@@ -1499,10 +1510,12 @@
 		} while (len);
 		close(ifd);
 		close(ofd);
+		set_mbgroup(outfile);
 		return 0;
 #ifdef HARDLINK_WHEN_POSSIBLE
 	} else {
 		/* Hard link succeeded */
+		set_mbgroup(outfile);
 		return 0;
 	}
 #endif
@@ -2642,6 +2655,7 @@
 				if (txt && EXISTS(dir,msgnum,fn,chan->language)) {
 					fclose(txt);
 					rename(tmptxtfile, txtfile);
+					set_mbgroup(txtfile);
 				} else if (txt && !EXISTS(dir,msgnum,fn,chan->language)) {
 					if (option_debug) 
 						ast_log(LOG_DEBUG, "The recorded media file is gone, so we should remove the .txt file too!\n");
@@ -2656,6 +2670,7 @@
 				fprintf(txt, "duration=%d\n", duration);
 				fclose(txt);
 				rename(tmptxtfile, txtfile);
+				set_mbgroup(txtfile);
 			}
 				
 			if (duration < vmminmessage) {
@@ -3550,6 +3565,7 @@
 				*/
 				snprintf(todir, sizeof(todir), "%s%s/%s/INBOX",  VM_SPOOL_DIR, vmtmp->context, vmtmp->mailbox);
 				snprintf(sys, sizeof(sys), "mkdir -p %s\n", todir);
+				set_mbgroup(todir);
 				snprintf(ext_context, sizeof(ext_context), "%s@%s", vmtmp->mailbox, vmtmp->context);
 				ast_log(LOG_DEBUG, "%s", sys);
 				ast_safe_system(sys);
@@ -6172,6 +6188,8 @@
 	char *exitcxt = NULL;	
 	char *extpc;
 	char *emaildateformatstr;
+	struct group *mailbox_group;
+	char *mbgroup;
 	int x;
 	int tmpadsi[4];
 
@@ -6578,6 +6596,19 @@
 				}
 				tmpread = tmpwrite+len;
 			}
+			/* define the group ownership of vm files for web access */
+			if ((mbgroup = ast_variable_retreive(cfg, "general", "voicemailgroup"))) {
+				mailbox_group = getgrnam(mbgroup);
+				if (mailbox_group == NULL) {
+					ast_log(LOG_WARNING, "Invalid voicemailgroup specified: %s\n" mbgroup);
+				} else { /* XXX Verify asterisk is a member XXX */
+					mbg = mailbox_group->gr_gid;
+					ast_log(LOG_DEBUG, "Found voicemailgroup: %s\n", mbgroup);
+				}
+			} else {
+				mailbox_group = getgrgid(getegid());
+				mbg = mailbox_group->gr_gid;			
+				ast_log(LOG_WARNING, "Using default group: %s\n", mailbox_group->gr_name)
 		}
 		AST_LIST_UNLOCK(&users);
 		ast_config_destroy(cfg);
